#!/usr/bin/perl
#----------------------------------------------------------------------
# heading     : Telephony
# description : network set IP
#
#
# Copyright (c) aelintra.com         2011
#
#----------------------------------------------------------------------
#----------------------------------------------------------------------
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
#
# Technical support for this program is available from Selintra Ltd
# Please visit our web site www.selintra.com/ for details.
#----------------------------------------------------------------------


use lib '/opt/sark/perl/modules';
use strict;
use sark::SarkSubs;

# Get ipaddress and netmask from user or system
my $ipaddr=shift;
my $nmask=shift;
unless ($ipaddr && $nmask) {
	$_ = `/sbin/ifconfig eth0`;
	/inet addr:*?([\d.]+)/;
	$ipaddr = $1;
	$_ = `/sbin/ifconfig eth0`;
	/Mask:*?([\d\.]+)/;
	$nmask = $1;
}

print "ipaddress is $ipaddr \n";
print "netmask is $nmask \n";

my @addrarr=split(/\./,$ipaddr);
my ( $ipaddress ) = unpack( "N", pack( "C4",@addrarr ) );

my @maskarr=split(/\./,$nmask);
my ( $netmask ) = unpack( "N", pack( "C4",@maskarr ) );

print "IP $ipaddress, MSK $netmask\n";

# Calculate network address by logical AND operation of addr & netmask
# and convert network address to IP address format
my $netadd = ( $ipaddress & $netmask );
my @netarr=unpack( "C4", pack( "N",$netadd ) );
my $netaddress=join(".",@netarr);

print "Network address : $netaddress \n";


# Calculate broadcase address by inverting the netmask
# and do a logical or with network address
my $bcast = ( $ipaddress & $netmask ) + ( ~ $netmask );
my @bcastarr=unpack( "C4", pack( "N",$bcast ) ) ;
my $broadcast=join(".",@bcastarr);

print "Broadcast address: $broadcast\n";

# Calculate CIDR by counting the bits in the mask 
my $nbits = 0;
$nmask =~ /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})/;
push (my @nums, $1, $2, $3, $4 );
foreach ( @nums ) {
	print "examining $_ \n";
	if ( /255/ ) {
		$nbits+=8;
	}
	elsif ( /254/ ) {
		$nbits+=7;
	} 
	elsif ( /252/ ) {
		$nbits+=6;
	} 
	elsif ( /248/ ) {
		$nbits+=5;
	} 
	elsif ( /240/ ) {
		$nbits+=4;
	} 
	elsif ( /224/ ) {
		$nbits+=3;
	} 
	elsif ( /192/ ) {
		$nbits+=2;
	}
	elsif ( /128/ ) {
		$nbits+=1;
	}  
}
print "Nbits is $nbits \n";
#open the database
`echo LAN=$netaddress/$nbits > /etc/shorewall/local.lan`;

# END of program

