#!/usr/bin/perl
# load required modules

use lib '/opt/sark/perl/modules';
use strict;
use POSIX qw(setsid);
use sark::SarkSubs;
use Net::Domain qw(hostname hostfqdn hostdomain);

my $t;
# flush the buffer
$| = 1;

#
# daemonize the program
#
&daemonize;
#
my $dbh = SarkSubs::SQLiteConnect();
my %global = SarkSubs::SQLiteGetHash($dbh, "SELECT * FROM globals WHERE pkey = 'global'");
SarkSubs::SQLiteDisconnect($dbh);

my $rank = "PRIMARY";
if ( $global{HASECNODE}  eq hostname() ) {
	$rank = "STANDBY";
}

print "$t SHADOW106 ==> Shadow is starting in passive mode\n";

while(1) {
#
#  Wait for asterisk to come up
#
	while ( ! `ps -A | grep asterisk | grep -v grep` ) {
		sleep(1);
	}
#
#  Check for Rhino SPF
#
if ($rank eq 'PRIMARY') {
	if ($global{RHINOSPF} eq 'YES') {
		if ( -e "/dev/ttyUSB0" ) {
                	`/usr/local/bin/rhinofailover /dev/ttyUSB0 RESET`;
                	sleep (1);
                	$t = localtime();
       			`/usr/local/bin/rhinofailover /dev/ttyUSB0 RELAYON 0`;
       			print "$t SHADOW102 ==> Rhino SPF set to NO(primary) mode\n";
        	}
		else {
			print "$t SHADOW301 ==> Globals states Rhino SPF but none found, turning off in globals\n";
                        $global{RHINOSPF} = 'NO';
                        $dbh = SarkSubs::SQLiteConnect();
                        SarkSubs::SQLiteDo($dbh, "UPDATE globals SET RHINOSPF='NO' WHERE pkey='global'");
                        SarkSubs::SQLiteDisconnect($dbh);
		}
	}

}

# start watchdog
	$t = localtime();
	print "$t SHADOW107 ==> Shadow is entering active mode\n";

	while(1) {
    		unless (`ps x | grep asterisk | grep -v grep`) {
			print "$t SHADOW100 ==> Asterisk is Dead!\n";
                        if ( SarkSubs::check_hapid() ) {
                                print "$t SHADOW105 ==> HA enabled - forcing failover\n";
				`/usr/lib/heartbeat/hb_standby`;
				if ($rank eq 'PRIMARY') {
					if ($global{RHINOSPF} eq 'YES') {
						if ( -e "/dev/ttyUSB0" ) {
                        				$t = localtime();
       							`/usr/local/bin/rhinofailover /dev/ttyUSB0 RELAYOFF 0`;
       							print "$t SHADOW202 ==> Rhino SPF set to NC(Secondary) mode\n";
						}
					}
    				}
                        	print "$t SHADOW106 ==> Shadow is entering passive mode\n";
				last;
                        }
                        elsif ( $global{RESTART} eq 'YES' ) {
                               print "$t SHADOW200 ==> No HA - Shadow will attempt Asterisk restart\n";
                               `/etc/init.d/sark stop`;
                               `/etc/init.d/dahdi stop`;
                               sleep 2;
                               `/etc/init.d/dahdi start`;
                               `/etc/init.d/sark start`;
                               print "$t SHADOW201 ==> Restart issued Shadow is entering passive mode\n";
			       last;
                        }
                        else {
                               print "$t SHADOW210 ==> No HA running and no restart enabled - Shadow is entering passive mode \n";
			       last;
                        }
    		}
    		sleep(1);
	}
}

sub daemonize {
    $t = localtime();
    chdir '/'                 or die "$t Can't chdir to /: $!";
    open STDIN, '/dev/null'   or die "$t Can't read /dev/null: $!";
    open STDOUT, '>>/var/log/shadow-log' or die "$t/tSHADOW102 ==> Can't write to log: $!\n";
    open STDERR, '>>/var/log/shadow-error-log' or die "$t/tSHADOW103 ==> Can't write to error log: $!\n";
    defined(my $pid = fork)   or die "$t SHADOW104==>Can't fork: $!";
    exit if $pid;
    setsid                    or die "$t SHADOWC105==>can't start a new session: $!\n";
    umask 0;
}