
#!/usr/bin/perl
#
#----------------------------------------------------------------------
# heading     : SAIL PBX
# description : srkdiag
# Copyright:  Aelintra Telecom Limited(2008-10), all rights reserved
#
#----------------------------------------------------------------------
#----------------------------------------------------------------------
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
#
# Technical support for this program is available from Aelintra Telecom Ltd
# Please visit our web site www.aelintra.com/ for details.
#----------------------------------------------------------------------
#
{

use strict;
use lib '/opt/sark/perl/modules';

use sark::SarkSubs;
use Net::Domain qw(hostname hostfqdn hostdomain);
use Sys::Syslog;

sub astcheck;
sub dahdicheck;
sub linecheck;
sub restart;
sub sendsms;
sub sendemail;

my $app_name = 'sarkdiag';
$0 = $app_name;


	openlog($0,'','user');

    my $tod = ();
    my @emails = ();
    my @numbers = ();
    my @diags = ();
    my $device = ();
    my $alarms = 0;

    my $dbh = SarkSubs::SQLiteConnect();
    my $notify = SarkSubs::SQLiteGet($dbh, "SELECT SMSALERT FROM globals where pkey = 'global'");
    my $path = SarkSubs::SQLiteGet($dbh, "SELECT SMSC FROM globals where pkey = 'global'");
    my $alert = SarkSubs::SQLiteGet($dbh, "SELECT ALERT FROM globals where pkey = 'global'");
    my $to =  SarkSubs::SQLiteGet($dbh, "SELECT EMAILALERT FROM globals where pkey = 'global'");
    my $attemptrestart =  SarkSubs::SQLiteGet($dbh, "SELECT ATTEMPTRESTART FROM globals where pkey = 'global'");
    SarkSubs::SQLiteDisconnect($dbh);

    my $sysname =  hostname();
    my $domname =  hostdomain();
    my $from = "admin\@$domname";
    my @rev = `/bin/cat /opt/sark/.sail-release`;
    my $subject = "SARK PBX System Alarm $sysname\@$domname";
    my $warnings = "SYSTEM NAME: $sysname... DOMAIN NAME: $domname... ";
    my 	$emailout;
    $emailout .= "The SARK UCS diagnostic routine has generated the following alerts...\n\n";
    $emailout .= "SYSTEM NAME: $sysname \nDOMAIN NAME: $domname \n\n";

    unless ( SarkSubs::check_hapid_installed ) {
    	&astcheck;
    }
    
    my $dbh = SarkSubs::SQLiteConnect();
    &linecheck;
	SarkSubs::SQLiteDisconnect($dbh);

    my $msg;
    if ($alarms){
        if (length($warnings) > 120) {
        	$msg = substr($warnings,0,119);
        }
        else {
        	$msg = $warnings;
        }
        $msg .= "Number of alarms = $alarms";

        if ($alert eq "sms" || $alert eq "both") {
        	@numbers = split (/ /, $notify);
                foreach (@numbers) {
    			sendsms ($msg,$_,$path);
                }
        }
        if ($alert eq "email" || $alert eq "both") {
        	@emails = split (/ /, $to);
                foreach (@emails) {
        		SarkSubs::sendemail ($_,$from,$subject,$emailout);
                }
        }
    }
    
    closelog();
    exit (0);



#
# SUBS BELOW HERE==>
#

sub astcheck  {
    unless ( SarkSubs::check_pid )  {
        if ( -e "/tmp/ael_txt_ok" ) {
                `rm -rf /tmp/ael_txt_ok`;
                $alarms++;
                if ($alert eq "email" || $alert eq "both") {
                        @emails = split (/\s+/, $to);
                        foreach (@emails) {
#                                print scalar(localtime)." $emailout SYS0001!! Asterisk is down!\n";
                                $emailout .= "\nSYSDIAG!! Asterisk is down!";
                                syslog('info', "Asterisk is down!");
                                
                        }
                }
                if  ($attemptrestart eq "YES") {
                        $emailout .= "\nSYSDIAG!! Attempting Restart";
                        &restart;
                }
        }
    }
    else {
        unless (  -e "/tmp/ael_txt_ok" ) {
                $alarms++;
                `/bin/touch /tmp/ael_txt_ok`;
                $emailout .= "\nSYSDIAG!! Asterisk operations resumed - clearing alarms\n";
                syslog('info', "Asterisk operations resumed - clearing alarms");
        }
    }
}


sub restart {

#
#	Attempt restart on non-HA systems
#
        if ( -e "/usr/lib/heartbeat/heartbeat" ) {
    		if (SarkSubs::check_hapid) {
                	return (0);
     		}
    	}

        syslog('info', "Issuing restart...");
        `/etc/init.d/sark stop`;
        `/etc/init.d/sark start`;
        unless ( SarkSubs::check_pid )  {
        	syslog('info', "Asterisk restart failed.");
        }
        else {
                syslog('info', "Asterisk restart Successful.");
        }
        return (0);
}

sub linecheck {

   my @Lines = sort (SarkSubs::SQLiteGetKeys($dbh, "SELECT lineIO.pkey FROM lineIO, Carrier "
    				."WHERE lineIO.carrier=Carrier.pkey "
                                ."AND NOT (carriertype='DiD' or carriertype='CLID' or carriertype='Class')"));

   foreach (@Lines) {
    	$device = $_;
		my %Line = SarkSubs::SQLiteGetHash($dbh, "SELECT * FROM lineIO WHERE pkey = '$_'");
    	if ($Line{monitor} eq 'YES') {
			my $technology = SarkSubs::SQLiteGet($dbh, "SELECT technology FROM Carrier where pkey = '$Line{carrier}'");
            if ($technology eq "SIP") {          
				@diags = `/usr/sbin/asterisk -rx "sip show peer $Line{peername}"`;
            }
            else {
				@diags = `/usr/sbin/asterisk -rx "iax2 show peer $Line{peername}"`;
			}	
            foreach (@diags) {
				if (/^\s*Status/) {
					unless (/OK/) {
						unless ( -e "/tmp/srkUA_$device" ) {
							$warnings .= "$technology/UA $device U/R... ";
                            $emailout .= "$technology/UA $device $_ \n";
                            `/bin/touch /tmp/srkUA_$device`;
                            $alarms++;
                            syslog('info', "$technology/UA $device $_");
                        }
                    }
                    else {
						if ( -e "/tmp/srkUA_$device" ) {	
							$warnings .= "$technology/UA $device U/R... ";
                            $warnings .= "$technology/UA $device Alarm Cleared ";
                            $emailout .= "$technology/UA $device Alarm Cleared \n";
                            $emailout .= "$technology/UA $device. $_ \n";
                            $alarms++;
                            syslog('info', "$technology/UA $device $_ alarm Cleared");            	
                        }
                        `/bin/rm -f /tmp/srkUA_$device`;
                    }
                }
            }
        }
   }
}

sub sendsms ($$$) {

    my ($msg, $notify, $path) = @_;
    my $message = "$notify $msg";
    my $rc = `smsq $message --motx-channel $path`;
    return ();
}

sub sendemail($$$$) {

    my ($to, $from, $subject, $content) = @_;
    my $sendmail = "/usr/sbin/sendmail -t";

    unless(open (MAIL, "|/usr/sbin/sendmail -t")) {
    	print "error.\n";
	warn "Error starting sendmail: $!";
        return (-1);
    }
    else {
	print MAIL "From: $from\n";
	print MAIL "To: $to\n";
	print MAIL "Subject: $subject\n\n";
	print MAIL $content;
	close(MAIL) || warn "Error closing mail: $!";
    }
    return();
}

sub dahdicheck {  
	
# function not implemented

my $tdmalarm = 'OFF';	
#   if ($dahdicheck) {
   	@diags = `/usr/sbin/asterisk -rx "dahdi show status"`;
        $device = $_;
        foreach (@diags) {
        	unless (/^Description/) {
                	unless (/OK/) {
                        	$tdmalarm = 'ON';
                                $warnings .= "TDMtrnk ".$device." U/R... ";
#                                print scalar(localtime)."  SRKDIAG - alarm $emailout\n";
                        }
                }
        }
#   }
   if ($tdmalarm eq 'OFF') {
	   if (-e "/opt/sark/alarms/tdmalarm") {
			`/bin/rm -f /opt/sark/alarms/$device`;
			$warnings .= "TDMtrnk Alarm Cleared... ";
#			print scalar(localtime)."  TDMtrnk Alarm Cleared...\n";
	   }
   }
}
}
